/*******************************************************************************
 * C H A M E L E O N   ColdFire C file                                         *
 *******************************************************************************
 * Copyright (C) 2001-2003 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/*******************************************************************************

	audio resampling application example

	The Coldfire side of the application responds to changes in the 
	Volume potentiometer and detects the state of the Edit key to
	activate/deactivate the bypass in the DSP. The data streams coming
	from the DSP through the host port are processed. Host communication
	follows a quite simplistic approach to the communications framework 
	implemented in the monosynth example.
	
*******************************************************************************/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <rtems.h>
#include <chameleon.h>

#include "dsp/dsp_code.h"

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

// set the appropiate values to kL and kM to display the working frequency
// properly

#define		kL			2	// Interpolation ratio for the input
#define		kM			3	// Decimation ratio for the input

#define		kChameleonSampleRate	48000
#define		kInternalSampleRate	(kChameleonSampleRate * kL / kM)

/******************************************************************************/

#define DSP_MAX_MESSAGE_LEN	32

/*******************************************************************************
	
	Global variables
	
	variables shared by the panel and dsp tasks, 
	
*******************************************************************************/

rtems_signed32		global_volume = 0; // written by the panel task, read by the dsp task
rtems_boolean		global_bypass = FALSE; // read and written by the panel task, read by the dsp task

/*******************************************************************************
	
	Auxiliary functions
	
*******************************************************************************/

void Error(char *error)
{
	TRACE(error);
	rtems_task_delete(RTEMS_SELF);
}

/*******************************************************************************

	PANEL TASK		

	Monitors the user actions on the front panel controls

*******************************************************************************/

static rtems_task panel_task(rtems_task_argument argument)
{
	int 			panel;

	rtems_unsigned8		potentiometer;
	rtems_unsigned8		value;
	rtems_unsigned32	key_bits;
	rtems_unsigned8		encoder;
	rtems_signed8		increment;
	
	int			i;
	float			dB;
	static rtems_unsigned32	volume_table[128];
	char			text[17];

	/* Initialization */

	TRACE("Starting panel_task...\n");

	/* Fill volume_table with a variable gain from -60 dB to 0dB */

	for (i = 0; i < 128; i++)
	{
		if (i < 27)
			dB = -90.0 + (float) 40.0*i/27.0;
		else
			dB = -50.0 + (float) 50.0*(i-27)/100.0;

		volume_table[i] = float_to_fix_round(pow(10.0, dB/20.0));
	}

	volume_table[0] = 0;

	panel = panel_init();

	if (!panel)
		Error("ERROR: cannot access the Panel.\n");

	panel_out_led(panel, 0);

	sprintf(text, "   %7d Hz   ", kInternalSampleRate);
	
	panel_out_lcd_print(panel, 0, 0, " Processing at: ");
	panel_out_lcd_print(panel, 1, 0, text);
	
	/* Endless loop scanning for user input */

	while (TRUE)
	{
		if (!panel_in_new_event(panel, TRUE))
			Error("ERROR: unexpected exit waiting new panel event.\n");

		if (panel_in_potentiometer(panel, &potentiometer, &value))
		{
			switch (potentiometer)
			{
				case PANEL01_POT_VOLUME:
					/* Volume change : causes the dsp_task to write the new value to the DSP */
					global_volume = volume_table[value];
					break;

				default:
					break;
			}
		}
		else if (panel_in_keypad(panel, &key_bits))
		{
			switch (key_bits)
			{
				case PANEL01_KEY_EDIT:
				{	
					/* Toggle internal sample rate */
					rtems_unsigned32 led;
					global_bypass = !global_bypass;
					
					if (global_bypass)
					{
						sprintf(text, "   %7d Hz   ", kChameleonSampleRate);
						led = PANEL01_LED_EDIT;
					}
					else
					{
						sprintf(text, "   %7d Hz   ", kInternalSampleRate);
						led = 0;
					}
					
					panel_out_lcd_print(panel, 1, 0, text);
					panel_out_led(panel, led);
				     	break;	
				}
				default:
					break;
			}
		}
		else if (panel_in_encoder(panel, &encoder, &increment))
		{
		}
	}

	/* Should never be here... */
	panel_exit(panel);
	Error("ERROR: Unexpected exit from panel_task");
}

/*******************************************************************************/

void create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('P', 'A', 'N', 'L'),
		30,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot create panel_task.\n");

	status = rtems_task_start(task_id, panel_task, 0);
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot start panel_task.\n");
}

/*******************************************************************************

	DSP TASKS
	
	read and write data from/to the DSP

*******************************************************************************/

static int dsp;

/******************************************************************************

 The same host message format as in monosynth is used

 DSP Message Header (1 DSP word):

	0xAABCCC

	AA  ( 8bits)	Total Message length (excluding header)
	B   ( 6bits)	Device	// not used
	CCC (10bits)	Message ID


******************************************************************************/

enum
{
	kDspWriteBlockP = 0x000,	// unused
	kDspWriteBlockX,		// unused
	kDspWriteBlockY,		// unused
	kDspWriteWordP,			// unused
	kDspWriteWordX,			// unused
	kDspWriteWordY,
	kDspCopyBlockP,			// unused
	kDspFillBlockX,			// unused
	kDspFillBlockY,			// unused

	///////////////////////////////////////////////////////

	kDspUsage = 0x300,			// 0x300..0x37F
	kDspTrace = 0x380,			// 0x380..0x3FF
};

/******************************************************************************/

void dsp_write_word_Y(rtems_signed32 addr, rtems_signed32 word)
{
	rtems_signed32 data[3];
	
	data[0] = ((2 & 0xFF) << 16) + ((0 & 0x3F)<<10) + (kDspWriteWordY & 0x3FF);
	data[1] = addr;
	data[2] = word;

	if (!dsp_write_data(dsp, data, 3))
		Error("ERROR: Cannot write data to the DSP in dsp_write_word_Y\n");
}

/******************************************************************************/

static rtems_task dsp_write_task(rtems_task_argument argument)
{
	rtems_signed32	volume = 0;
	rtems_boolean	bypass = FALSE;

	TRACE("Starting dsp_write_task...\n");

	while (TRUE)
	{
		// Check for changes in the Volume and Bypass and update the DSP
	
		if (volume != global_volume)
		{
			volume = global_volume;
			dsp_write_word_Y(DSPY_MasterVolumeTarget,volume);
		}
		else if (bypass != global_bypass)
		{
			bypass = global_bypass;

			if (bypass)
				dsp_write_word_Y(DSPY_BypassEnabledTarget, 1);
			else
				dsp_write_word_Y(DSPY_BypassEnabledTarget, 0);
		}
	}
	
	/* Should never be here... */
	dsp_exit(dsp);
	Error("ERROR: Unexpected exit from dsp_write_task");
}

/******************************************************************************/

void create_dsp_write_task(void)
{
	rtems_id	  	task_id;
	rtems_status_code 	status;

	status = rtems_task_create(
		rtems_build_name('T', 'D', 'S', 'W'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot create dsp_write_task\n");

	status = rtems_task_start(task_id, dsp_write_task, 0);
	
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot start dsp_write_task\n");
}

/******************************************************************************/

static rtems_task dsp_read_task(rtems_task_argument argument)
{
	rtems_unsigned32 data_count;
	rtems_unsigned32 message_id;
	rtems_signed32   data[DSP_MAX_MESSAGE_LEN];

	TRACE("Starting dsp_read_read...\n");

	while (TRUE)
	{
		// check for incoming DSP data and process it
	
		while (!dsp_read_data(dsp, data, 1))
			continue;

		data_count = (data[0] >> 16) & 0xFF; 	//GetDataCount(data[0]);
		message_id = (data[0] & 0x3FF); 	//GetMessageID(data[0]);
		
		if (data_count)
		{
			ASSERT(data_count <= DSP_MAX_MESSAGE_LEN);
			while (!dsp_read_data(dsp, data+1, data_count))
				continue;
		}
		
		if ((message_id & 0x380) == kDspTrace)
		{
			char ch = (message_id & 0x7F);
			
			if (ch == 0x0A)
				TRACE("\n");
			else if (ch >= 0x20)
				TRACE("%c", (char) ch);
		}
		else if ((message_id & 0x380) == kDspUsage)
		{
			TRACE("DSP usage: %d%%\n", (BYTE) (message_id & 0x7F));
		}
	}
	
	/* Should never be here... */
	dsp_exit(dsp);
	Error("ERROR: Unexpected exit from dsp_read_task");
}

/******************************************************************************/

void create_dsp_read_task(void)
{
	rtems_id	  	task_id;
	rtems_status_code 	status;

	status = rtems_task_create(
		rtems_build_name('T', 'D', 'S', 'R'),
		25,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot create dsp_read_task\n");

	status = rtems_task_start(task_id, dsp_read_task, 0);
	
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot start dsp_read_task\n");
}

/******************************************************************************/

void create_dsp_tasks()
{
	dsp = dsp_init(1, dspCode);

	if (dsp < 0)
		Error("ERROR: Cannot access the dsp");

	create_dsp_read_task();
	create_dsp_write_task();
}

/*******************************************************************************

	MAIN TASK		

	Creates the panel and dsp tasks and deletes itself

*******************************************************************************/

rtems_task rtems_main(rtems_task_argument ignored)
{
	TRACE("******************************************************************************\n");
	TRACE("Audio resampling example\n");
	TRACE("******************************************************************************\n");

	create_panel_task();
	create_dsp_tasks();
	
	rtems_task_delete(RTEMS_SELF);
}

/*******************************************************************************/