//******************************************************************************
// C H A M E L E O N    S. D. K.                                               *
//******************************************************************************
//  $Archive:: /Chameleon.sdk/src/examples/resample/scilab/resfilt.sci 	       $
//     $Date::                                                                 $
// $Revision:: 1                                                               $
//-----------------------------------------------------------------------------*
// This file is part of the Chameleon Software Development Kit                 *
//                                                                             *
// Copyright (C) 2001-2003 Soundart                                            *
// www.soundart-hot.com                                                        *
// support@soundart-hot.com                                                    *
//*****************************************************************************/

///////////////////////////////////////////////////////////////////////////////

function h = resfilt(L, M, At, NCoeffs)

// function h = resfilt(L, M, [, At, NCoeffs])
//
// creates a Kaiser window Low Pass filter for the ratio L/M resampling algorithm
// parameters:
// 	L : Interpolation ratio (must be integer)
// 	M : Decimation ratio (must be integer)
//	At: Stopband attenuation in dB. If not specified, it is set to 80 dB as default
//	NCoeffs: number of coefficients of the Kaiser Low Pass filter. If is not specified,
//	an optimum number is estimated.
//
//	h : vector containing the filter coefficients

	if (argn(2) == 2)
		NCoeffs = -1;
		At = 80;
	else
		if (argn(2) == 3)
			NCoeffs = -1;
		else
			if (argn(2) ~= 4)
				error('Wrong number of input args');
			end;
		end;
	end;


	Ratio = L / M;

	[alpha, alphanorm] = getalpha(At);

	if (Ratio >= 1)
		fc = 0.5;
	else
		fc = 0.5 * Ratio;
	end;

	Gain = L;

	if (NCoeffs == -1)
		n = ceil((L * alphanorm) / (0.15 * fc)) + 1;
		m = ceil((n - 1) / (2 * L));
		NCoeffs = 2 * m * L + 1;
	end;

	span = NCoeffs - 1;

	del = 0.5 * span;

	h = KaiserLPF(fc / L, NCoeffs, alpha, Gain, span);

endfunction

///////////////////////////////////////////////////////////////////////////////

function [alpha, alphanorm] = getalpha(At)

	select At
		case 30 then
			alpha = 2.1166248611410;
			alphanorm = 1.5355227584626;

		case 40 then
			alpha = 3.3953210522615;
			alphanorm = 2.2318824619304;

		case 50 then
			alpha = 4.5335141209812;
			alphanorm = 2.9170589564815;

		case 60 then
			alpha = 5.6532600000000;
			alphanorm = 3.6246518105850;

		case 70 then
			alpha = 6.7552600000000;
			alphanorm = 4.3210306406685;

		case 80 then
			alpha = 7.8572600000000;
			alphanorm = 5.0174094707521;

		case 90 then
			alpha = 8.9592600000000;
			alphanorm = 5.7137883008357;

		case 100 then
			alpha = 10.061260000000;
			alphanorm = 6.4101671309192;

		else
			// default to 80 dB
			alpha = 7.85726;
			alphanorm = 5.0174094707521;
	end;

endfunction

///////////////////////////////////////////////////////////////////////////////

function h = KaiserLPF(fc, N, alpha, Gain, span)

	Ga = 2 * fc * Gain;
	T = 0.5 * span;

	h = zeros(N, 1);
	k = N;

	for i=1:k,
		t = i - T;
		u = k - T;

		h(i) = Ga * NSinc(2 * fc * (t - 1)) * kaiserwin((t - 1) / T, alpha);

		if (-t == u)
			h(k) = h(i);
		else
			h(k) = Ga * NSinc(2 * fc * (u - 1)) * kaiserwin((u - 1) / T, alpha);
		end;

		k = k - 1;
	end;

endfunction

///////////////////////////////////////////////////////////////////////////////

function y = NSinc(x)

	if (x == 0)
		y = 1;
	else
		xm = modulo(x,2);
		if (x == floor(x))
			y = 0;
		else
			y = sin(%pi * xm) / (%pi * x);
		end
	end;

endfunction

///////////////////////////////////////////////////////////////////////////////

function w = kaiserwin(x, alpha)

	if ((x < -1) | (x > 1))
		w = 0;
	else
		Ia = besseli(0, alpha);	// 1st kind bessel I0 function
		beta = alpha * sqrt(1 - x*x);
		w = besseli(0, beta) / Ia;
	end;

endfunction

///////////////////////////////////////////////////////////////////////////////

function hp = getpolyphase(Ir, h)

	// transform the filter h into its polyphase form (Ir phases)

	// remove trailing zeros

	while (h(length(h)) == 0)
		h = h(1:(length(h)-1));
	end;

	len = length(h);
	N = ceil(len/Ir);

	if modulo(len, Ir) ~= 0
		for i=1:(Ir-1)
			h(len + 1) = 0;
			len = len + 1;
		end;
	end;

	hp = zeros(N, Ir);

	for j=1:Ir,
		for i=1:N
			hp(i,j) = h(Ir*i - ( Ir - j));
		end;
	end;

endfunction

///////////////////////////////////////////////////////////////////////////////

function [sel, lim] = getpolyselector(L, M)

	// polyphase selectors and their limits for the inner loop

	sel = zeros(L, 1);	// polyphase selectors

	for i=0:(L - 1),
		sel(i + 1) = modulo(M * i, L);
	end;

	lim = zeros(M, 2);	// polyphase selector limits for the inner loop

	for i = 0:(M - 1),

		lim(i + 1, 1) = ceil(L * i / M);

		lim(i + 1, 2) = floor(L * (i + 1) / M);

		if ((lim(i + 1, 2)) > (L - 1))
			lim(i + 1, 2) = L - 1;
		end;
	end;

endfunction

///////////////////////////////////////////////////////////////////////////////